@@ -639,7 +639,7 @@ Enquire::Internal::set_query(const Query &query_, termcount qlen_)
 }
 
 const Query &
-Enquire::Internal::get_query()
+Enquire::Internal::get_query() const
 {
     return query;
 }
@@ -892,12 +892,7 @@ const Query &
 Enquire::get_query() const
 {
     LOGCALL(API, const Xapian::Query &, "Xapian::Enquire::get_query", NO_ARGS);
-    try {
-	RETURN(internal->get_query());
-    } catch (Error & e) {
-	if (internal->errorhandler) (*internal->errorhandler)(e);
-	throw;
-    }
+    RETURN(internal->get_query());
 }
 
 void
