@@ -48,6 +48,7 @@
 #include "brass_values.h"
 #include "debuglog.h"
 #include "io_utils.h"
+#include "net/length.h"
 #include "pack.h"
 #include "remoteconnection.h"
 #include "replication.h"
@@ -1106,7 +1107,7 @@ BrassWritableDatabase::add_document(const Xapian::Document & document)
 {
     LOGCALL(DB, Xapian::docid, "BrassWritableDatabase::add_document", document);
     // Make sure the docid counter doesn't overflow.
-    if (stats.get_last_docid() == Xapian::docid(-1))
+    if (stats.get_last_docid() == BRASS_MAX_DOCID)
 	throw Xapian::DatabaseError("Run out of docids - you'll have to use copydatabase to eliminate any gaps before you can add more documents");
     // Use the next unused document ID.
     RETURN(add_document_(stats.get_next_docid(), document));
