@@ -1,7 +1,7 @@
 /** @file postingsource.h
  *  @brief External sources of posting information
  */
-/* Copyright (C) 2007,2008,2009,2010,2011,2012 Olly Betts
+/* Copyright (C) 2007,2008,2009,2010,2011,2012,2015 Olly Betts
  * Copyright (C) 2008,2009 Lemur Consulting Ltd
  *
  * This program is free software; you can redistribute it and/or modify
@@ -51,8 +51,14 @@ class XAPIAN_VISIBILITY_DEFAULT PostingSource {
     void * matcher_;
 
   protected:
+#ifdef SWIG
+  public: /* So SWIG wraps this constructor correctly for PHP */
+#endif
     /// Allow subclasses to be instantiated.
     PostingSource() : max_weight_(0), matcher_(NULL) { }
+#ifdef SWIG
+  protected:
+#endif
 
     /** Set an upper bound on what get_weight() can return from now on.
      *
@@ -382,7 +388,7 @@ class XAPIAN_VISIBILITY_DEFAULT ValuePostingSource : public PostingSource {
      *
      *  @param slot_ The value slot to read values from.
      */
-    ValuePostingSource(Xapian::valueno slot_);
+    explicit ValuePostingSource(Xapian::valueno slot_);
 
     Xapian::doccount get_termfreq_min() const;
     Xapian::doccount get_termfreq_est() const;
@@ -425,7 +431,7 @@ class XAPIAN_VISIBILITY_DEFAULT ValueWeightPostingSource
      *
      *  @param slot_ The value slot to read values from.
      */
-    ValueWeightPostingSource(Xapian::valueno slot_);
+    explicit ValueWeightPostingSource(Xapian::valueno slot_);
 
     Xapian::weight get_weight() const;
     ValueWeightPostingSource * clone() const;
@@ -514,7 +520,7 @@ class XAPIAN_VISIBILITY_DEFAULT ValueMapPostingSource
      *
      *  @param slot_ The value slot to read values from.
      */
-    ValueMapPostingSource(Xapian::valueno slot_);
+    explicit ValueMapPostingSource(Xapian::valueno slot_);
 
     /** Add a mapping.
      *
@@ -569,7 +575,7 @@ class XAPIAN_VISIBILITY_DEFAULT FixedWeightPostingSource : public PostingSource
      *
      *  @param wt The fixed weight to return.
      */
-    FixedWeightPostingSource(Xapian::weight wt);
+    explicit FixedWeightPostingSource(Xapian::weight wt);
 
     Xapian::doccount get_termfreq_min() const;
     Xapian::doccount get_termfreq_est() const;
