@@ -130,7 +130,7 @@ class XAPIAN_VISIBILITY_DEFAULT StringValueRangeProcessor : public ValueRangePro
      *
      *  @param slot_	The value number to return from operator().
      */
-    StringValueRangeProcessor(Xapian::valueno slot_)
+    explicit StringValueRangeProcessor(Xapian::valueno slot_)
 	: valno(slot_), str() { }
 
     /** Constructor.
@@ -312,7 +312,7 @@ class XAPIAN_VISIBILITY_DEFAULT NumberValueRangeProcessor : public StringValueRa
      *
      *  @param slot_   The value number to return from operator().
      */
-    NumberValueRangeProcessor(Xapian::valueno slot_)
+    explicit NumberValueRangeProcessor(Xapian::valueno slot_)
 	: StringValueRangeProcessor(slot_) { }
 
     /** Constructor.
@@ -389,7 +389,9 @@ class XAPIAN_VISIBILITY_DEFAULT QueryParser {
 	FLAG_LOVEHATE = 4,
 	/// Support AND, OR, etc even if they aren't in ALLCAPS.
 	FLAG_BOOLEAN_ANY_CASE = 8,
-	/** Support right truncation (e.g. Xap*).
+	/** Support wildcards.
+	 *
+	 *  At present only right truncation (e.g. Xap*) is supported.
 	 *
 	 *  Currently you can't use wildcards with boolean filter prefixes,
 	 *  or in a phrase (either an explicitly quoted one, or one implicitly
@@ -459,6 +461,20 @@ class XAPIAN_VISIBILITY_DEFAULT QueryParser {
 	 */
 	FLAG_AUTO_MULTIWORD_SYNONYMS = 1024 | FLAG_AUTO_SYNONYMS,
 
+	/** Enable generation of n-grams from CJK text.
+	 *
+	 *  With this enabled, spans of CJK characters are split into unigrams
+	 *  and bigrams, with the unigrams carrying positional information.
+	 *  Non-CJK characters are split into words as normal.
+	 *
+	 *  The corresponding option needs to have been used at index time.
+	 *
+	 *  Flag added in Xapian 1.3.4 and 1.2.22, but this mode can be
+	 *  enabled in 1.2.8 and later by setting environment variable
+	 *  XAPIAN_CJK_NGRAM.
+	 */
+	FLAG_CJK_NGRAM = 2048,
+
 	/** The default flags.
 	 *
 	 *  Used if you don't explicitly pass any to @a parse_query().
@@ -574,13 +590,13 @@ class XAPIAN_VISIBILITY_DEFAULT QueryParser {
      *		   calling get_msg() on the caught exception.  The current
      *		   possible values (in case you want to translate them) are:
      *
-     *			@li Unknown range operation
-     *			@li parse error
-     *			@li Syntax: &lt;expression&gt; AND &lt;expression&gt;
-     *			@li Syntax: &lt;expression&gt; AND NOT &lt;expression&gt;
-     *			@li Syntax: &lt;expression&gt; NOT &lt;expression&gt;
-     *			@li Syntax: &lt;expression&gt; OR &lt;expression&gt;
-     *			@li Syntax: &lt;expression&gt; XOR &lt;expression&gt;
+     *		   @li Unknown range operation
+     *		   @li parse error
+     *		   @li Syntax: &lt;expression&gt; AND &lt;expression&gt;
+     *		   @li Syntax: &lt;expression&gt; AND NOT &lt;expression&gt;
+     *		   @li Syntax: &lt;expression&gt; NOT &lt;expression&gt;
+     *		   @li Syntax: &lt;expression&gt; OR &lt;expression&gt;
+     *		   @li Syntax: &lt;expression&gt; XOR &lt;expression&gt;
      */
     Query parse_query(const std::string &query_string,
 		      unsigned flags = FLAG_DEFAULT,
