@@ -1,7 +1,7 @@
 /** @file weight.h
  * @brief Weighting scheme API.
  */
-/* Copyright (C) 2007,2008,2009,2010,2011,2012 Olly Betts
+/* Copyright (C) 2007,2008,2009,2010,2011,2012,2014,2015 Olly Betts
  * Copyright (C) 2009 Lemur Consulting Ltd
  *
  * This program is free software; you can redistribute it and/or
@@ -34,17 +34,29 @@ class XAPIAN_VISIBILITY_DEFAULT Weight {
   protected:
     /// Stats which the weighting scheme can use (see @a need_stat()).
     typedef enum {
+	/// Number of documents in the collection.
 	COLLECTION_SIZE = 1,
+	/// Number of documents in the RSet.
 	RSET_SIZE = 2,
+	/// Average length of documents in the collection.
 	AVERAGE_LENGTH = 4,
+	/// How many documents the current term is in.
 	TERMFREQ = 8,
+	/// How many documents in the RSet the current term is in.
 	RELTERMFREQ = 16,
+	/// Sum of wqf for terms in the query.
 	QUERY_LENGTH = 32,
+	/// Within-query-frequency of the current term.
 	WQF = 64,
+	/// Within-document-frequency of the current term in the current document.
 	WDF = 128,
+	/// Length of the current document (sum wdf).
 	DOC_LENGTH = 256,
+	/// Lower bound on (non-zero) document lengths.
 	DOC_LENGTH_MIN = 512,
+	/// Upper bound on document lengths.
 	DOC_LENGTH_MAX = 1024,
+	/// Upper bound on wdf.
 	WDF_MAX = 2048
     } stat_flags;
 
@@ -270,9 +282,15 @@ class XAPIAN_VISIBILITY_DEFAULT Weight {
      */
     Weight(const Weight &);
 
+#ifdef SWIG
+  public: /* So SWIG wraps this constructor correctly for PHP */
+#endif
     /// Default constructor, needed by subclass constructors.
     Weight() : stats_needed() { }
 
+#ifdef SWIG
+  protected:
+#endif
     /// The number of documents in the collection.
     Xapian::doccount get_collection_size() const { return collection_size_; }
 
