@@ -2,7 +2,7 @@
  * @brief Combine subqueries, weighting as if they are synonyms
  */
 /* Copyright 2007,2009 Lemur Consulting Ltd
- * Copyright 2009 Olly Betts
+ * Copyright 2009,2014 Olly Betts
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -26,6 +26,7 @@
 
 #include "branchpostlist.h"
 #include "debuglog.h"
+#include "omassert.h"
 
 SynonymPostList::~SynonymPostList()
 {
@@ -78,9 +79,14 @@ SynonymPostList::get_weight() const
 
     if (want_wdf) {
 	Xapian::termcount wdf = get_wdf();
-	Xapian::termcount doclen = get_doclength();
-	if (wdf > doclen) wdf = doclen;
-	RETURN(wt->get_sumpart(wdf, doclen));
+	Xapian::termcount doclen = 0;
+	if (want_doclength || wdf > doclen_lower_bound) {
+	    doclen = get_doclength();
+	    if (wdf > doclen) wdf = doclen;
+	}
+	double sumpart = wt->get_sumpart(wdf, doclen);
+	AssertRel(sumpart, <=, wt->get_maxpart());
+	RETURN(sumpart);
     }
     RETURN(wt->get_sumpart(0, want_doclength ? get_doclength() : 0));
 }
