@@ -34,6 +34,7 @@
 
 #include "autoptr.h"
 #include "multimatch.h"
+#include "net/length.h"
 #include "omassert.h"
 #include "realtime.h"
 #include "serialise.h"
@@ -117,16 +118,21 @@ RemoteServer::get_message(double timeout, string & result,
 			  message_type required_type)
 {
     double end_time = RealTime::end_time(timeout);
-    unsigned int type = RemoteConnection::get_message(result, end_time);
+    int type = RemoteConnection::get_message(result, end_time);
 
-    // Handle "shutdown connection" message here.
-    if (type == MSG_SHUTDOWN) throw ConnectionClosed();
+    // Handle "shutdown connection" message here.  Treat EOF here for a read-only
+    // database the same way since a read-only client just closes the
+    // connection when done.
+    if (type == MSG_SHUTDOWN || (type < 0 && wdb == NULL))
+	throw ConnectionClosed();
+    if (type < 0)
+	throw Xapian::NetworkError("Connection closed unexpectedly");
     if (type >= MSG_MAX) {
 	string errmsg("Invalid message type ");
 	errmsg += str(type);
 	throw Xapian::NetworkError(errmsg);
     }
-    if (required_type != MSG_MAX && type != unsigned(required_type)) {
+    if (required_type != MSG_MAX && type != int(required_type)) {
 	string errmsg("Expecting message type ");
 	errmsg += str(int(required_type));
 	errmsg += ", got ";
@@ -251,7 +257,8 @@ RemoteServer::msg_termlist(const string &message)
 {
     const char *p = message.data();
     const char *p_end = p + message.size();
-    Xapian::docid did = decode_length(&p, p_end, false);
+    Xapian::docid did;
+    decode_length(&p, p_end, did);
 
     send_message(REPLY_DOCLENGTH, encode_length(db->get_doclength(did)));
     const Xapian::TermIterator end = db->termlist_end(did);
@@ -270,7 +277,8 @@ RemoteServer::msg_positionlist(const string &message)
 {
     const char *p = message.data();
     const char *p_end = p + message.size();
-    Xapian::docid did = decode_length(&p, p_end, false);
+    Xapian::docid did;
+    decode_length(&p, p_end, did);
     string term(p, p_end - p);
 
     Xapian::termpos lastpos = static_cast<Xapian::termpos>(-1);
@@ -367,20 +375,23 @@ RemoteServer::msg_query(const string &message_in)
 {
     const char *p = message_in.c_str();
     const char *p_end = p + message_in.size();
-    size_t len;
 
     // Unserialise the Query.
-    len = decode_length(&p, p_end, true);
+    size_t len;
+    decode_length_and_check(&p, p_end, len);
     AutoPtr<Xapian::Query::Internal> query(Xapian::Query::Internal::unserialise(string(p, len), reg));
     p += len;
 
     // Unserialise assorted Enquire settings.
-    Xapian::termcount qlen = decode_length(&p, p_end, false);
+    Xapian::termcount qlen;
+    decode_length(&p, p_end, qlen);
 
-    Xapian::valueno collapse_max = decode_length(&p, p_end, false);
+    Xapian::valueno collapse_max;
+    decode_length(&p, p_end, collapse_max);
 
     Xapian::valueno collapse_key = Xapian::BAD_VALUENO;
-    if (collapse_max) collapse_key = decode_length(&p, p_end, false);
+    if (collapse_max)
+	decode_length(&p, p_end, collapse_key);
 
     if (p_end - p < 4 || *p < '0' || *p > '2') {
 	throw Xapian::NetworkError("bad message (docid_order)");
@@ -388,7 +399,8 @@ RemoteServer::msg_query(const string &message_in)
     Xapian::Enquire::docid_order order;
     order = static_cast<Xapian::Enquire::docid_order>(*p++ - '0');
 
-    Xapian::valueno sort_key = decode_length(&p, p_end, false);
+    Xapian::valueno sort_key;
+    decode_length(&p, p_end, sort_key);
 
     if (*p < '0' || *p > '3') {
 	throw Xapian::NetworkError("bad message (sort_by)");
@@ -412,7 +424,7 @@ RemoteServer::msg_query(const string &message_in)
     }
 
     // Unserialise the Weight object.
-    len = decode_length(&p, p_end, true);
+    decode_length_and_check(&p, p_end, len);
     string wtname(p, len);
     p += len;
 
@@ -425,19 +437,19 @@ RemoteServer::msg_query(const string &message_in)
 					   wtname + " not registered");
     }
 
-    len = decode_length(&p, p_end, true);
+    decode_length_and_check(&p, p_end, len);
     AutoPtr<Xapian::Weight> wt(wttype->unserialise(string(p, len)));
     p += len;
 
     // Unserialise the RSet object.
-    len = decode_length(&p, p_end, true);
+    decode_length_and_check(&p, p_end, len);
     Xapian::RSet rset = unserialise_rset(string(p, len));
     p += len;
 
     // Unserialise any MatchSpy objects.
     MatchSpyList matchspies;
     while (p != p_end) {
-	len = decode_length(&p, p_end, true);
+	decode_length_and_check(&p, p_end, len);
 	string spytype(p, len);
 	const Xapian::MatchSpy * spyclass = reg.get_match_spy(spytype);
 	if (spyclass == NULL) {
@@ -446,7 +458,7 @@ RemoteServer::msg_query(const string &message_in)
 	}
 	p += len;
 
-	len = decode_length(&p, p_end, true);
+	decode_length_and_check(&p, p_end, len);
 	matchspies.spies.push_back(spyclass->unserialise(string(p, len), reg));
 	p += len;
     }
@@ -464,11 +476,13 @@ RemoteServer::msg_query(const string &message_in)
     p = message.c_str();
     p_end = p + message.size();
 
-    Xapian::termcount first = decode_length(&p, p_end, false);
-    Xapian::termcount maxitems = decode_length(&p, p_end, false);
+    Xapian::termcount first;
+    decode_length(&p, p_end, first);
+    Xapian::termcount maxitems;
+    decode_length(&p, p_end, maxitems);
 
-    Xapian::termcount check_at_least = 0;
-    check_at_least = decode_length(&p, p_end, false);
+    Xapian::termcount check_at_least;
+    decode_length(&p, p_end, check_at_least);
 
     message.erase(0, message.size() - (p_end - p));
     Xapian::Weight::Internal total_stats(unserialise_stats(message));
@@ -496,17 +510,19 @@ RemoteServer::msg_query_new(const string &message_in)
     size_t len;
 
     // Unserialise the Query.
-    len = decode_length(&p, p_end, true);
+    decode_length_and_check(&p, p_end, len);
     AutoPtr<Xapian::Query::Internal> query(Xapian::Query::Internal::unserialise(string(p, len), reg));
     p += len;
 
     // Unserialise assorted Enquire settings.
-    Xapian::termcount qlen = decode_length(&p, p_end, false);
+    Xapian::termcount qlen;
+    decode_length(&p, p_end, qlen);
 
-    Xapian::valueno collapse_max = decode_length(&p, p_end, false);
+    Xapian::valueno collapse_max;
+    decode_length(&p, p_end, collapse_max);
 
     Xapian::valueno collapse_key = Xapian::BAD_VALUENO;
-    if (collapse_max) collapse_key = decode_length(&p, p_end, false);
+   if (collapse_max) decode_length(&p, p_end, collapse_key);
 
     if (p_end - p < 4 || *p < '0' || *p > '2') {
 	throw Xapian::NetworkError("bad message (docid_order)");
@@ -514,7 +530,8 @@ RemoteServer::msg_query_new(const string &message_in)
     Xapian::Enquire::docid_order order;
     order = static_cast<Xapian::Enquire::docid_order>(*p++ - '0');
 
-    Xapian::valueno sort_key = decode_length(&p, p_end, false);
+    Xapian::valueno sort_key;
+    decode_length(&p, p_end, sort_key);
 
     if (*p < '0' || *p > '3') {
 	throw Xapian::NetworkError("bad message (sort_by)");
@@ -538,7 +555,7 @@ RemoteServer::msg_query_new(const string &message_in)
     }
 
     // Unserialise the Weight object.
-    len = decode_length(&p, p_end, true);
+    decode_length_and_check(&p, p_end, len);
     string wtname(p, len);
     p += len;
 
@@ -551,19 +568,19 @@ RemoteServer::msg_query_new(const string &message_in)
 					   wtname + " not registered");
     }
 
-    len = decode_length(&p, p_end, true);
+    decode_length_and_check(&p, p_end, len);
     AutoPtr<Xapian::Weight> wt(wttype->unserialise(string(p, len)));
     p += len;
 
     // Unserialise the RSet object.
-    len = decode_length(&p, p_end, true);
+    decode_length_and_check(&p, p_end, len);
     Xapian::RSet rset = unserialise_rset(string(p, len));
     p += len;
 
     // Unserialise any MatchSpy objects.
     MatchSpyList matchspies;
     while (p != p_end) {
-	len = decode_length(&p, p_end, true);
+	decode_length_and_check(&p, p_end, len);
 	string spytype(p, len);
 	const Xapian::MatchSpy * spyclass = reg.get_match_spy(spytype);
 	if (spyclass == NULL) {
@@ -572,7 +589,7 @@ RemoteServer::msg_query_new(const string &message_in)
 	}
 	p += len;
 
-	len = decode_length(&p, p_end, true);
+	decode_length_and_check(&p, p_end, len);
 	matchspies.spies.push_back(spyclass->unserialise(string(p, len), reg));
 	p += len;
     }
@@ -590,11 +607,13 @@ RemoteServer::msg_query_new(const string &message_in)
     p = message.c_str();
     p_end = p + message.size();
 
-    Xapian::termcount first = decode_length(&p, p_end, false);
-    Xapian::termcount maxitems = decode_length(&p, p_end, false);
+    Xapian::termcount first;
+    decode_length(&p, p_end, first);
+    Xapian::termcount maxitems;
+    decode_length(&p, p_end, maxitems);
 
-    Xapian::termcount check_at_least = 0;
-    check_at_least = decode_length(&p, p_end, false);
+    Xapian::termcount check_at_least;
+    decode_length(&p, p_end, check_at_least);
 
     message.erase(0, message.size() - (p_end - p));
     Xapian::Weight::Internal total_stats(unserialise_stats(message));
@@ -619,7 +638,8 @@ RemoteServer::msg_document(const string &message)
 {
     const char *p = message.data();
     const char *p_end = p + message.size();
-    Xapian::docid did = decode_length(&p, p_end, false);
+    Xapian::docid did;
+    decode_length(&p, p_end, did);
 
     Xapian::Document doc = db->get_document(did);
 
@@ -666,7 +686,8 @@ RemoteServer::msg_valuestats(const string & message)
     const char *p = message.data();
     const char *p_end = p + message.size();
     while (p != p_end) {
-	Xapian::valueno slot = decode_length(&p, p_end, false);
+	Xapian::valueno slot;
+	decode_length(&p, p_end, slot);
 	string message_out;
 	message_out += encode_length(db->get_value_freq(slot));
 	string bound = db->get_value_lower_bound(slot);
@@ -685,7 +706,8 @@ RemoteServer::msg_doclength(const string &message)
 {
     const char *p = message.data();
     const char *p_end = p + message.size();
-    Xapian::docid did = decode_length(&p, p_end, false);
+    Xapian::docid did;
+    decode_length(&p, p_end, did);
     send_message(REPLY_DOCLENGTH, encode_length(db->get_doclength(did)));
 }
 
@@ -731,7 +753,8 @@ RemoteServer::msg_deletedocument(const string & message)
 
     const char *p = message.data();
     const char *p_end = p + message.size();
-    Xapian::docid did = decode_length(&p, p_end, false);
+    Xapian::docid did;
+    decode_length(&p, p_end, did);
 
     wdb->delete_document(did);
 
@@ -755,7 +778,8 @@ RemoteServer::msg_replacedocument(const string & message)
 
     const char *p = message.data();
     const char *p_end = p + message.size();
-    Xapian::docid did = decode_length(&p, p_end, false);
+    Xapian::docid did;
+    decode_length(&p, p_end, did);
 
     wdb->replace_document(did, unserialise_document(string(p, p_end)));
 }
@@ -768,7 +792,8 @@ RemoteServer::msg_replacedocumentterm(const string & message)
 
     const char *p = message.data();
     const char *p_end = p + message.size();
-    size_t len = decode_length(&p, p_end, true);
+    size_t len;
+    decode_length_and_check(&p, p_end, len);
     string unique_term(p, len);
     p += len;
 
@@ -802,7 +827,8 @@ RemoteServer::msg_setmetadata(const string & message)
 	throw Xapian::InvalidOperationError("Server is read-only");
     const char *p = message.data();
     const char *p_end = p + message.size();
-    size_t keylen = decode_length(&p, p_end, false);
+    size_t keylen;
+    decode_length_and_check(&p, p_end, keylen);
     string key(p, keylen);
     p += keylen;
     string val(p, p_end - p);
@@ -816,7 +842,8 @@ RemoteServer::msg_addspelling(const string & message)
 	throw Xapian::InvalidOperationError("Server is read-only");
     const char *p = message.data();
     const char *p_end = p + message.size();
-    Xapian::termcount freqinc = decode_length(&p, p_end, false);
+    Xapian::termcount freqinc;
+    decode_length(&p, p_end, freqinc);
     wdb->add_spelling(string(p, p_end - p), freqinc);
 }
 
@@ -827,6 +854,7 @@ RemoteServer::msg_removespelling(const string & message)
 	throw Xapian::InvalidOperationError("Server is read-only");
     const char *p = message.data();
     const char *p_end = p + message.size();
-    Xapian::termcount freqdec = decode_length(&p, p_end, false);
+    Xapian::termcount freqdec;
+    decode_length(&p, p_end, freqdec);
     wdb->remove_spelling(string(p, p_end - p), freqdec);
 }
