@@ -76,6 +76,12 @@ DEFINE_TESTCASE(totaldoclen1, writable) {
     TEST_EQUAL(db.get_avlength(), 2000000000);
     db.commit();
     TEST_EQUAL(db.get_avlength(), 2000000000);
+    for (int i = 0; i != 20; ++i) {
+	db.add_document(doc);
+    }
+    TEST_EQUAL(db.get_avlength(), 2000000000);
+    db.commit();
+    TEST_EQUAL(db.get_avlength(), 2000000000);
     if (get_dbtype() != "inmemory") {
 	// InMemory doesn't support get_writable_database_as_database().
 	Xapian::Database dbr = get_writable_database_as_database();
@@ -93,18 +99,63 @@ DEFINE_TESTCASE(dbstats1, backend) {
     const Xapian::termcount max_len = 532;
     const Xapian::termcount max_wdf = 22;
 
-    if (get_dbtype().find("chert") != string::npos ||
-	get_dbtype().find("brass") != string::npos) {
-	// Should be exact for brass and chert as no deletions have happened.
+    if (get_dbtype() != "inmemory" &&
+	get_dbtype().find("flint") == string::npos) {
+	// Should be exact as no deletions have happened.
 	TEST_EQUAL(db.get_doclength_upper_bound(), max_len);
 	TEST_EQUAL(db.get_doclength_lower_bound(), min_len);
     } else {
-	// For other backends, we usually give rather loose bounds.
+	// For flint and inmemory, we usually give rather loose bounds.
 	TEST_REL(db.get_doclength_upper_bound(),>=,max_len);
 	TEST_REL(db.get_doclength_lower_bound(),<=,min_len);
     }
 
-    TEST_REL(db.get_wdf_upper_bound("the"),>=,max_wdf);
+    if (get_dbtype() != "inmemory" && !startswith(get_dbtype(), "remote") &&
+	get_dbtype().find("flint") == string::npos) {
+	TEST_EQUAL(db.get_wdf_upper_bound("the"), max_wdf);
+    } else {
+	// For flint, inmemory and remote backends, we usually give rather
+	// loose bounds (remote matches use tighter bounds, but querying the
+	// wdf bound gives a looser one).
+	TEST_REL(db.get_wdf_upper_bound("the"),>=,max_wdf);
+    }
+
+    return true;
+}
+
+// Check stats with a single document.  In a multi-database situation, this
+// gave 0 for get-_doclength_lower_bound() in 1.3.2.
+DEFINE_TESTCASE(dbstats2, backend) {
+    Xapian::Database db = get_database("apitest_onedoc");
+
+    // Use precalculated values to avoid expending CPU cycles to calculate
+    // these every time without improving test coverage.
+    const Xapian::termcount min_len = 15;
+    const Xapian::termcount max_len = 15;
+    const Xapian::termcount max_wdf = 7;
+
+    if (get_dbtype() != "inmemory" &&
+	get_dbtype().find("flint") == string::npos) {
+	// Should be exact as no deletions have happened.
+	TEST_EQUAL(db.get_doclength_upper_bound(), max_len);
+	TEST_EQUAL(db.get_doclength_lower_bound(), min_len);
+    } else {
+	// For inmemory and flint, we usually give rather loose bounds.
+	TEST_REL(db.get_doclength_upper_bound(),>=,max_len);
+	TEST_REL(db.get_doclength_lower_bound(),<=,min_len);
+    }
+
+    if (get_dbtype() != "inmemory" && get_dbtype() != "flint" &&
+	!startswith(get_dbtype(), "remote")) {
+	TEST_EQUAL(db.get_wdf_upper_bound("word"), max_wdf);
+    } else {
+	// For inmemory, flint, and remote backends, we usually give rather
+	// loose bounds (remote matches use tighter bounds, but querying the
+	// wdf bound gives a looser one).
+	TEST_REL(db.get_wdf_upper_bound("word"),>=,max_wdf);
+    }
+
+    TEST_EQUAL(db.get_wdf_upper_bound(""), 0);
 
     return true;
 }
@@ -948,12 +999,12 @@ DEFINE_TESTCASE(phrasebug1, generated && positional) {
     Xapian::Database db = get_database("phrasebug1", make_phrasebug1_db);
     const char * qterms[] = { "katrina", "hurricane" };
     Xapian::Enquire e(db);
-    Xapian::Query q(q.OP_PHRASE, qterms, qterms + 2, 5);
+    Xapian::Query q(Xapian::Query::OP_PHRASE, qterms, qterms + 2, 5);
     e.set_query(q);
     Xapian::MSet mset = e.get_mset(0, 100);
     TEST_EQUAL(mset.size(), 0);
     const char * qterms2[] = { "hurricane", "katrina" };
-    Xapian::Query q2(q.OP_PHRASE, qterms2, qterms2 + 2, 5);
+    Xapian::Query q2(Xapian::Query::OP_PHRASE, qterms2, qterms2 + 2, 5);
     e.set_query(q2);
     mset = e.get_mset(0, 100);
     TEST_EQUAL(mset.size(), 1);
@@ -996,7 +1047,7 @@ DEFINE_TESTCASE(dbfilefd012, brass || chert) {
 #else
 		// If we can't check it is a socket, at least check it is not a
 		// regular file.
-		TEST(!S_ISREG(sb.st_mode);
+		TEST(!S_ISREG(sb.st_mode));
 #endif
 	    }
 	}
@@ -1078,3 +1129,11 @@ DEFINE_TESTCASE(sortvalue2, backend) {
     }
     return true;
 }
+
+/// Check behaviour of Enquire::get_query().
+DEFINE_TESTCASE(enquiregetquery1, backend) {
+    Xapian::Database db = get_database("apitest_simpledata");
+    Xapian::Enquire enq(db);
+    TEST_EQUAL(enq.get_query().get_description(), "Xapian::Query()");
+    return true;
+}
