@@ -1,7 +1,8 @@
-/* testutils.h: Xapian-specific test helper functions and macros.
- *
- * Copyright 1999,2000,2001 BrightStation PLC
- * Copyright 2002,2003,2007,2008 Olly Betts
+/** @file testutils.h
+ * @brief Xapian-specific test helper functions and macros.
+ */
+/* Copyright 1999,2000,2001 BrightStation PLC
+ * Copyright 2002,2003,2007,2008,2015 Olly Betts
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -29,7 +30,7 @@
 // Useful display operators
 
 std::ostream &operator<<(std::ostream &os,
-			 const std::vector<unsigned int> &ints);
+			 const std::vector<Xapian::docid> &ints);
 
 // ######################################################################
 // Useful comparison operators
