@@ -37,6 +37,7 @@ using namespace std;
 #include "testsuite.h"
 #include "testutils.h"
 
+#include "net/length.h"
 #include "omassert.h"
 #include "omqueryinternal.h"
 #include "pack.h"
@@ -44,6 +45,13 @@ using namespace std;
 #include "serialise-double.h"
 #include "str.h"
 
+#if defined XAPIAN_ENABLE_VISIBILITY && defined __GNUC__ && (__GNUC__ >= 4)
+# define XAPIAN_UNITTEST
+# include "../net/length.cc"
+#else
+# define Xapian_NetworkError Xapian::NetworkError
+#endif
+
 static bool test_except1()
 {
     try {
@@ -260,7 +268,8 @@ static bool test_serialiselength1()
 	string s = encode_length(n);
 	const char *p = s.data();
 	const char *p_end = p + s.size();
-	size_t decoded_n = decode_length(&p, p_end, false);
+	size_t decoded_n;
+	decode_length(&p, p_end, decoded_n);
 	if (n != decoded_n || p != p_end) tout << "[" << s << "]" << endl;
 	TEST_EQUAL(n, decoded_n);
 	TEST_EQUAL(p_end - p, 0);
@@ -283,66 +292,84 @@ static bool test_serialiselength2()
 	{
 	    const char *p = s.data();
 	    const char *p_end = p + s.size();
-	    TEST(decode_length(&p, p_end, true) == 0);
+	    unsigned result;
+	    decode_length_and_check(&p, p_end, result);
+	    TEST(result == 0);
 	    TEST(p == p_end);
 	}
 	s += 'x';
 	{
 	    const char *p = s.data();
 	    const char *p_end = p + s.size();
-	    TEST(decode_length(&p, p_end, true) == 0);
+	    unsigned result;
+	    decode_length_and_check(&p, p_end, result);
+	    TEST(result == 0);
 	    TEST_EQUAL(p_end - p, 1);
 	}
     }
     // Special case tests for 1
     {
 	string s = encode_length(1);
-	TEST_EXCEPTION(Xapian::NetworkError,
+	TEST_EXCEPTION(Xapian_NetworkError,
 	    const char *p = s.data();
 	    const char *p_end = p + s.size();
-	    TEST(decode_length(&p, p_end, true) == 1);
+	    unsigned result;
+	    decode_length_and_check(&p, p_end, result);
+	    TEST(result == 1);
 	);
 	s += 'x';
 	{
 	    const char *p = s.data();
 	    const char *p_end = p + s.size();
-	    TEST(decode_length(&p, p_end, true) == 1);
+	    unsigned result;
+	    decode_length_and_check(&p, p_end, result);
+	    TEST(result == 1);
 	    TEST_EQUAL(p_end - p, 1);
 	}
 	s += 'x';
 	{
 	    const char *p = s.data();
 	    const char *p_end = p + s.size();
-	    TEST(decode_length(&p, p_end, true) == 1);
+	    unsigned result;
+	    decode_length_and_check(&p, p_end, result);
+	    TEST(result == 1);
 	    TEST_EQUAL(p_end - p, 2);
 	}
     }
     // Nothing magic here, just test a range of odd and even values.
     for (size_t n = 2; n < 1000; n = (n + 1) * 2 + (n >> 1)) {
 	string s = encode_length(n);
-	TEST_EXCEPTION(Xapian::NetworkError,
+	TEST_EXCEPTION(Xapian_NetworkError,
 	    const char *p = s.data();
 	    const char *p_end = p + s.size();
-	    TEST(decode_length(&p, p_end, true) == n);
+	    unsigned result;
+	    decode_length_and_check(&p, p_end, result);
+	    TEST(result == n);
 	);
 	s.append(n-1, 'x');
-	TEST_EXCEPTION(Xapian::NetworkError,
+	TEST_EXCEPTION(Xapian_NetworkError,
 	    const char *p = s.data();
 	    const char *p_end = p + s.size();
-	    TEST(decode_length(&p, p_end, true) == n);
+	    unsigned result;
+	    decode_length_and_check(&p, p_end, result);
+	    TEST(result == n);
 	);
 	s += 'x';
 	{
 	    const char *p = s.data();
 	    const char *p_end = p + s.size();
-	    TEST(decode_length(&p, p_end, true) == n);
+	    unsigned result;
+	    decode_length_and_check(&p, p_end, result);
+	    TEST(result == n);
 	    TEST_EQUAL(size_t(p_end - p), n);
 	}
 	s += 'x';
 	{
 	    const char *p = s.data();
 	    const char *p_end = p + s.size();
-	    TEST(decode_length(&p, p_end, true) == n);
+	    unsigned result;
+	    decode_length_and_check(&p, p_end, result);
+	    TEST(result == n);
 	    TEST_EQUAL(size_t(p_end - p), n + 1);
 	}
     }
